%
% MATLAB SCRIPT TO GENERATE INPUT MODES FOR CFD BASED EIGENVALUE 
% FLUTTER ANALYSIS
%
% MODES ARE GENERATED BASED ON 7 RANDOMIZED PARAMETERS
% FOR THE GOLAND WING AS DEFINED BY MSC.NASTRAN F.E. MODEL
% AVAILABLE AT: http://www.cfd4aircraft.com/models/goland/goland_clean_103.bdf
%
% MSC. NASTRAN INPUT DECK USES A SEPARATE FILE WITH VALUES FOR 
% UNCERTAIN PARAMETERS (PARM.DAT)
%
clc
clear all
%
global ncases
%
%setting paths
%nastran files are expected to be in current directory
path1   = [pwd '/'];
cd(path1);
%
%mean parameters description and input conditions
MU      = [0.0155;0.0155;0.0006;0.0006;0.0416;0.1496;0.0416];
SIGMA   = (0.05*diag(MU)).^2;
ncases  = 1000;
np      = 7;
noVar   = 7;
inputVarM =mvnrnd(MU,SIGMA,ncases);
savefile1 ='parameters.mat';
save (savefile1,'inputVarM');
vflutter=zeros(ncases,1);
con3=0;

%GENERATING MODE FILES FOR MC ANALYSIS
for itr=1:ncases

    updatingvalue = inputVarM(itr,:);
    
    namefiletowrite = ['PARM.DAT'];
    x   = [path1,namefiletowrite];
    fid = fopen(x,'w');

    fprintf(fid,'PSHELL*  220000          1               %2.4e\n',updatingvalue(1));
    fprintf(fid,'PSHELL*  230000          1               %2.4e\n',updatingvalue(2));
    fprintf(fid,'PSHELL*  240000          1               %2.4e\n',updatingvalue(3));
    fprintf(fid,'PSHELL*  240100          1               %2.4e\n',updatingvalue(4));
    fprintf(fid,'PROD*    120000          1               %2.4e\n',updatingvalue(5));
    fprintf(fid,'PROD*    120100          1               %2.4e\n',updatingvalue(6));
    fprintf(fid,'PROD*    120200          1               %2.4e\n',updatingvalue(7));
    fclose(fid);
    %
    %saving parameter file
    namefiletowrite = ['PARM_' num2str(itr) '.DAT'];
    x   = [path1,namefiletowrite];
    fid = fopen(x,'w');
    fprintf(fid,'PSHELL*  220000          1               %2.4e\n',updatingvalue(1));
    fprintf(fid,'PSHELL*  230000          1               %2.4e\n',updatingvalue(2));
    fprintf(fid,'PSHELL*  240000          1               %2.4e\n',updatingvalue(3));
    fprintf(fid,'PSHELL*  240100          1               %2.4e\n',updatingvalue(4));
    fprintf(fid,'PROD*    120000          1               %2.4e\n',updatingvalue(5));
    fprintf(fid,'PROD*    120100          1               %2.4e\n',updatingvalue(6));
    fprintf(fid,'PROD*    120200          1               %2.4e\n',updatingvalue(7));
    fclose(fid);
    %
    unix(['msc2006 nastran goland_clean_103.bdf scr=yes batch=no']);
    pause (1);
%
    namefiletoopen =['goland_clean_103.f06'];
    y = [path1,namefiletoopen];
    fid=fopen(y);
    str='R E A L   E I G E N V A L U E S';
    len_str=length(str);

    found = 0;
    while found==0
        tline = fgetl(fid);
        for(t=1:length(tline)-length(str)+1)
            countmatch = 0;
            for(count=1:length(str))
                if(str(count)==tline(t+count-1))
                    countmatch=countmatch+1;
                end
            end
            if countmatch==len_str
                found =1;
                break,end
        end
    end
    
    tline = fgetl(fid);
    tline = fgetl(fid);
    a = fscanf(fid, '%g',[7 8]);

    %%%%%%%%% retrieving modes %%%%%%%%%%

    namefiletoopen =['phi.op4'];
    %pathname= [pwd '/'];
    y = [path1,namefiletoopen];
    fid = fopen(y);
    status=fseek(fid,0,'eof');EOF=ftell(fid);currentFPI=fseek(fid,0,'bof');
    % reading MX size in first line
    line=fgetl(fid);currentFPI=ftell(fid);
    matrixdata=str2num(line(1:8*2));
    PHI=zeros(fliplr(matrixdata));
    while currentFPI<EOF
        % reading data position
        line=fgetl(fid);currentFPI=ftell(fid);
        mxPosition=str2num(line);
        if mxPosition(2)<=size(PHI,1) & mxPosition(1)<=size(PHI,2) % it always gives extra useless number in num_col+1
            counter=1;clear mxData
            while counter<=mxPosition(3)
                line=fgetl(fid);currentFPI=ftell(fid);
                newline=double(line);
                cc=0;
                for ijk=16:16:length(newline)-1;
                    if newline(ijk+1)~=double(' ')
                        newline=[newline(1:ijk+cc) double(' ') newline(ijk+1+cc:end)];
                        cc=cc+1;
                    end
                end
                line=char(newline);
                data=str2num(line);
                if length(data)==5
                    mxData(counter:counter+4)=data;
                    counter=counter+5;
                else
                    mxData(counter:counter+length(data)-1)=data;
                    counter=counter+length(data);
                end
            end
            PHI(mxPosition(2):mxPosition(2)+mxPosition(3)-1,mxPosition(1))=mxData';
        else
            break
        end
    end
    fclose (fid);
    for num=1:30
        MOD1(num,:)=[PHI(19+6*num,1) PHI(20+6*num,1) PHI(21+6*num,1)];
        MOD2(num,:)=[PHI(19+6*num,2) PHI(20+6*num,2) PHI(21+6*num,2)];
        MOD3(num,:)=[PHI(19+6*num,3) PHI(20+6*num,3) PHI(21+6*num,3)];
        MOD4(num,:)=[PHI(19+6*num,4) PHI(20+6*num,4) PHI(21+6*num,4)];
        MOD5(num,:)=[PHI(19+6*num,5) PHI(20+6*num,5) PHI(21+6*num,5)];
    end
    namefiletowrite = ['clean' num2str(itr) '.modes'];
    x1 = [path1,namefiletowrite];
    fidup = fopen(x1,'w');
    
    if norm (MOD4(:,3))>norm (MOD5(:,3))
        y=[a(5,1:4)]';
    else
        MOD4=MOD5;
        y=[a(5,1:3) a(5,5)]';
    end
    
    fprintf(fidup, '4\n');
    fprintf(fidup, '33\n');
    fprintf(fidup, '6.0\n');
    fprintf(fidup, '%8.6E 0. 0. 0.\n', y);
    fprintf(fidup,'% 2.1f        %2.1f          %2.1f\n',zeros (3));
    fprintf(fidup,'% 8.6E     %8.6E  %8.6E\n',MOD1');
    fprintf(fidup,'% 2.1f        %2.1f          %2.1f\n',zeros (3));
    fprintf(fidup,'% 8.6E     %8.6E  %8.6E\n',MOD2');
    fprintf(fidup,'% 2.1f        %2.1f          %2.1f\n',zeros (3));
    fprintf(fidup,'% 8.6E     %8.6E  %8.6E\n',MOD3');
    fprintf(fidup,'% 2.1f        %2.1f          %2.1f\n',zeros (3));
    fprintf(fidup,'% 8.6E     %8.6E  %8.6E\n',MOD4');
    fclose (fidup);
    unix(['rm -f *.f0* *.xdb *.log *.pch phi.op4 *.becho* *.asm.*']);
%
    !zip params.zip -9 -m PARM_*.zip
%
end
%